@@ -1,5 +1,6 @@
 # infra
-from .interface import AsyncRunnable, LazyLoadable, MessageTypes, ChannelTypes, ChannelPrivacyTypes
+from .interface import AsyncRunnable, LazyLoadable
+from .interface import MessageTypes, ChannelTypes, ChannelPrivacyTypes, EventTypes
 from .cert import Cert
 from .receiver import Receiver, WebhookReceiver, WebsocketReceiver
 from .requester import HTTPRequester
@@ -9,7 +10,7 @@ from .client import Client
 # concepts
 from .role import Role
 from .user import User
-from .channel import Channel, PublicTextChannel, PrivateChannel
+from .channel import Channel, PublicTextChannel, PrivateChannel, PublicChannel, public_channel_factory
 from .guild import Guild
 from .context import Context
 from .message import RawMessage, Message, PublicMessage, PrivateMessage, Event
