@@ -3,7 +3,7 @@ from abc import ABC, abstractmethod
 from typing import Union, List, overload, Dict
 
 from . import api
-from .gateway import Requestable
+from .gateway import Requestable, Gateway
 from .interface import LazyLoadable, MessageTypes, ChannelTypes
 
 
@@ -11,8 +11,6 @@ class Channel(LazyLoadable, Requestable, ABC):
     """
     Interface, represents a channel where messages flowing
     """
-
-    id: str
     type: ChannelTypes
 
     @abstractmethod
@@ -22,6 +20,11 @@ class Channel(LazyLoadable, Requestable, ABC):
         """
         raise NotImplementedError
 
+    @property
+    @abstractmethod
+    def id(self) -> str:
+        raise NotImplementedError
+
 
 class PublicChannel(Channel, ABC):
     name: str
@@ -36,7 +39,7 @@ class PublicChannel(Channel, ABC):
     permission_sync: int
 
     def __init__(self, **kwargs):
-        self.id: str = kwargs.get('id')
+        self._id: str = kwargs.get('id')
         self.name: str = kwargs.get('name')
         self.user_id: str = kwargs.get('user_id')
         self.guild_id: str = kwargs.get('guild_id')
@@ -52,6 +55,10 @@ class PublicChannel(Channel, ABC):
         self._loaded = kwargs.get('_lazy_loaded_', False)
         self.gate = kwargs.get('_gate_')
 
+    @property
+    def id(self) -> str:
+        return self._id
+
 
 class PublicTextChannel(PublicChannel):
     """
@@ -80,12 +87,16 @@ class PublicTextChannel(PublicChannel):
         """
         # if content is card msg, then convert it to plain str
         if isinstance(content, List):
-            kwargs['type'] = MessageTypes.CARD.value
+            kwargs['type'] = MessageTypes.CARD
             content = json.dumps(content)
+        if 'type' not in kwargs:
+            kwargs['type'] = MessageTypes.TEXT
 
         # merge params
         kwargs['target_id'] = self.id
         kwargs['content'] = content
+        if isinstance(kwargs['type'], MessageTypes):
+            kwargs['type'] = kwargs['type'].value
         if temp_target_id:
             kwargs['temp_target_id'] = temp_target_id
 
@@ -109,6 +120,14 @@ class PublicVoiceChannel(PublicChannel):
         raise TypeError('now there is no PublicVoiceChannel, *hey dude we have a pkg from future*')
 
 
+def public_channel_factory(_gate_: Gateway, **kwargs) -> PublicChannel:
+    kwargs['type'] = kwargs['type'] if isinstance(kwargs['type'], ChannelTypes) else ChannelTypes(kwargs['type'])
+    if kwargs['type'] == ChannelTypes.TEXT:
+        return PublicTextChannel(**kwargs, _gate_=_gate_)
+    elif kwargs['type'] == ChannelTypes.VOICE:
+        return PublicVoiceChannel(**kwargs, _gate_=_gate_)
+
+
 class PrivateChannel(Channel):
     """
     Private chat channel
@@ -139,6 +158,10 @@ class PrivateChannel(Channel):
     async def load(self):
         pass
 
+    @property
+    def id(self) -> str:
+        return self.target_user_id
+
     @property
     def target_user_id(self) -> str:
         return self.target_info.get('id') if self.target_info else None
@@ -160,6 +183,8 @@ class PrivateChannel(Channel):
         if isinstance(content, List):
             kwargs['type'] = MessageTypes.CARD.value
             content = json.dumps(content)
+        if 'type' not in kwargs:
+            kwargs['type'] = MessageTypes.TEXT
 
         # merge params
         if self.code:
@@ -167,5 +192,7 @@ class PrivateChannel(Channel):
         else:
             kwargs['target_id'] = self.target_user_id
         kwargs['content'] = content
+        if isinstance(kwargs['type'], MessageTypes):
+            kwargs['type'] = kwargs['type'].value
 
         return await self.gate.exec_req(api.DirectMessage.create(**kwargs))
